import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max,
} from 'd3';

// Select the SVG element from the DOM
const svg = select('svg');

// Get the width and height of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Function to render the line chart
const render = (data) => {
  // Title for the chart
  const title = 'US Textile Total Sales from 2004 to 2024';

  // Function to get x-value from data
  const xValue = (d) => d.Year;
  const xAxisLabel = 'Year'; // Label for x-axis

  // Function to get y-value from data
  const yValue = (d) => d.TotalSales;
  const yAxisLabel = 'Total Sales in Million'; // Label for y-axis

  // Margin object to provide spacing around the chart
  const margin = {
    top: 80,
    right: 40,
    bottom: 88,
    left: 150,
  };
  // Calculate inner width and height
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Append a 'g' element for positioning the chart content
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Create x-scale using scaleTime
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  // Create x-axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Create y-scale using scaleLinear
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();

  // Create y-axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  // Append y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -90)
    .attr('x', -innerHeight / 3)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .text(yAxisLabel);

  // Append x-axis to the chart
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 70)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

  // Create a line generator
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)));

  // Append path element for the line
  g.append('path')
    .attr('class', 'line_path')
    .attr('d', lineGenerator(data));

  // Append circles for each data point
  g.selectAll('.dot')
    .data(data)
    .enter()
    .append('circle')
    .attr('class', 'dot')
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('r', 5);

  // Append title for the chart
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);
};

// Fetch data from CSV file
csv(
  'https://gist.githubusercontent.com/akhilcharugulla/d314cfc95b0d87ecc59768a5d8c9a018/raw/ddccf3b1d133c3391f0663ecb780814693b1c616/textile.csv',
).then((data) => {
  // Convert TotalSales to numeric and Year to Date object
  data.forEach((d) => {
    d.TotalSales = +d.TotalSales;
    d.Year = new Date(d.Year);
  });
  // Call the render function with the fetched data
  render(data);
});
